               //THIS CODE IS FOR THE CAN DEVICE DAUGHTERBOARD

#define CAN_BITRATE             1000000

#define CAN_RX_FIFO_SIZE 4



/* TYPE DEFINITIONS */
struct CANData{
	char cID;
	unsigned long ulData;
};
typedef struct CANData tCANData;

/* GLOBAL VARIABLES */
unsigned long Motor1Data, Motor2Data, Motor3Data, Motor4Data; 

unsigned char ucPingData[PING_DATA_LENGTH];
unsigned char ucRot1Data[ROT1_DATA_LENGTH];
unsigned char ucRot2Data[ROT2_DATA_LENGTH];


//*****************************************************************************
//
// The CAN controller interrupt handler.
//
//*****************************************************************************
void CANIntHandler(void){
    unsigned long ulIntStatus, ulIDStatus;
	int i;
	tCANMsgObject xTempMsgObject;
	tCANData xTempCANData;
	unsigned char ucData[8];

	xTempMsgObject.pucMsgData = ucData;

    // Find the cause of the interrupt, if it is a status interrupt then just
    // acknowledge the interrupt by reading the status register.
    ulIntStatus = CANIntStatus(CAN0_BASE, CAN_INT_STS_CAUSE);
	
	//check to see if a receive or other status interrupt caused the interrupt
	if (ulIntStatus & CAN_INT_INTID_STATUS){
		ulIDStatus = CANStatusGet(CAN0_BASE, CAN_STS_NEWDAT);
		for (i = 0; i < 32; i++){
			//test every bit of the mask, if true, perform operation
			if ( (0x1 << i) & ulIDStatus){
				//if this one has an interrupt, get the data
				CANMessageGet(CAN0_BASE, (i+1), &xTempMsgObject, TRUE);
				xTempCANData.cID = (char)xTempMsgObject.ulMsgID;
				xTempCANData.ulData = (xTempMsgObject.pucMsgData[0] << 0) | (xTempMsgObject.pucMsgData[1] << 8) | (xTempMsgObject.pucMsgData[2] << 16) | (xTempMsgObject.pucMsgData[3] << 24);
				//put the data into a global or FIFO, depennding upon how you want to buffer the data
				switch (xTempCANData.cID){
					case MOTOR1_ID: Motor1Data = xTempCANData.ulData; break;
					case MOTOR2_ID: Motor2Data = xTempCANData.ulData; break;
					case MOTOR3_ID: Motor3Data = xTempCANData.ulData; break;
					case MOTOR4_ID: Motor4Data = xTempCANData.ulData; break;
					//if the receive didn't match a buffer, you screwed up and just dump the data.
					default: break;
				}
			}
		}
	}
    // Acknowledge the CAN controller interrupt has been handled.
    CANIntClear(CAN0_BASE, ulIntStatus);
}

void CAN_Open(void){
	if(REVISION_IS_A2){
        SysCtlLDOSet(SYSCTL_LDO_2_75V);
    }
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
    GPIOPinTypeCAN(GPIO_PORTD_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);
    GPIOPinTypeGPIOOutput(GPIO_PORTF_BASE, GPIO_PIN_2);
    GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_2, 0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_CAN0);
    CANInit(CAN0_BASE);
    CANBitRateSet(CAN0_BASE, 8000000, CAN_BITRATE);
    CANEnable(CAN0_BASE);
	//make sure to enable STATUS interrupts
    CANIntEnable(CAN0_BASE, CAN_INT_MASTER | CAN_INT_ERROR | CAN_INT_STATUS);
    IntEnable(INT_CAN0);
    return;
}

//Set up a message object.  Can be a TX object or an RX object.
void CAN_Setup_Message_Object(	unsigned long MessageID, \
								unsigned long MessageFlags, \
								unsigned long MessageLength, \
								unsigned char * MessageData, \
								unsigned long ObjectID, \
								tMsgObjType eMsgType){
	tCANMsgObject xTempObject;
	xTempObject.ulMsgID = MessageID;
	xTempObject.ulMsgLen = MessageLength;
	xTempObject.pucMsgData = MessageData;
	xTempObject.ulFlags = MessageFlags;
	CANMessageSet(CAN0_BASE, ObjectID, &xTempObject, eMsgType);
}

//create all the receive objects for the lab. Must be a dual of the other board's transmit objects.
int CAN_Create_RX_Message_Objects(void){
	//set up all the receive objects for the motor receive commands
	CAN_Setup_Message_Object(MOTOR1_ID, MSG_OBJ_RX_INT_ENABLE, MOTOR1_DATA_LENGTH, NULL, MOTOR1_ID, MSG_OBJ_TYPE_RX);
	CAN_Setup_Message_Object(MOTOR2_ID, MSG_OBJ_RX_INT_ENABLE, MOTOR2_DATA_LENGTH, NULL, MOTOR2_ID, MSG_OBJ_TYPE_RX);
	CAN_Setup_Message_Object(MOTOR3_ID, MSG_OBJ_RX_INT_ENABLE, MOTOR3_DATA_LENGTH, NULL, MOTOR3_ID, MSG_OBJ_TYPE_RX);
	CAN_Setup_Message_Object(MOTOR4_ID, MSG_OBJ_RX_INT_ENABLE, MOTOR4_DATA_LENGTH, NULL, MOTOR4_ID, MSG_OBJ_TYPE_RX);
	return TRUE;
}

//send some data to test
void test (void){
	for(;;){
		//create message object for the PING sensor
		CAN_Setup_Message_Object( PING_ID, NULL, PING_DATA_LENGTH, ucPingData, PING_ID, MSG_OBJ_TYPE_TX);
		CAN_Setup_Message_Object( ROT1_ID, NULL, PING_DATA_LENGTH, ucRot1Data, ROT1_ID, MSG_OBJ_TYPE_TX);
		CAN_Setup_Message_Object( ROT2_ID, NULL, PING_DATA_LENGTH, ucRot2Data, ROT2_ID, MSG_OBJ_TYPE_TX);
		OS_Sleep(1000);
	}
}


//Code from MAIN
CAN_Open();
CAN_Create_RX_Message_Objects();