// ****CanTest.c************
// Runs on LM3S8962 <===> LM3S2110
// Test of CAN driver
// Jonathan Valvano, drived from code written by Donald Owen
// December 9, 2011
// The system communicates 4-byte objects
// The UP switch (SW3) on the LM3S8962 is shown on the STATUS LED of LM3S2110
// The UP switch (SW100) on the LM3S2110 is shown on the STATUS LED of LM3S8962
// The DOWN switch (SW101) on the LM3S2110 is shown on the green Ethernet LED of LM3S8962
// The yellow Ethernet LED of LM3S8962 flashes when received a CAN input
// PD0/CAN0Rx
// PD1/CAN0Tx

/* This example accompanies the book
   Embedded Systems: Real-Time Operating Systems for the Arm Cortex-M3, Volume 3,  
   ISBN: 978-1466468863, Jonathan Valvano, copyright (c) 2012

   Program xx.x, section xx.x

 Copyright 2012 by Jonathan W. Valvano, valvano@mail.utexas.edu
    You may use, edit, run or distribute this file
    as long as the above copyright notice remains
 THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 VALVANO SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 For more information about my classes, my research, and my books, see
 http://users.ece.utexas.edu/~valvano/
 */
#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_can.h"
#include "inc/hw_types.h"
#include "driverlib/can.h"
#include "driverlib/debug.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"

#include "CAN0.h"

void DisableInterrupts(void); // Disable interrupts
void EnableInterrupts(void);  // Enable interrupts
long StartCritical (void);    // previous I bit, disable interrupts
void EndCritical(long sr);    // restore I bit to previous value
void WaitForInterrupt(void);  // low power mode
/* GLOBAL VARIABLES */

#ifdef __TI_COMPILER_VERSION__
	//Code Composer Studio
	// each AND instruction is 1 cycle
	void Delay(void){
	__asm(	"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     and r0,r0\n"
			"     bx  R14\n");
}

#else
	//Keil uVision
	__asm void
	Delay(void){
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     and r0,r0
     bx  R14
}
#endif

unsigned char XmtData[4];
unsigned char RcvData[4];
unsigned long RcvCount=0;
void ReceiveData(void){
  if(CAN0_GetMailNonBlock(RcvData)){
    RcvCount++;
    GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_0, RcvData[0]);
    // PF2 is negative logic, data arrives as 0 or 2
    GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_2, 0x04^(RcvData[1]<<1));
    GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_3, RcvCount);   // heartbeat
  } 
}
void Delay1ms(int t){ int i;
  while(t){
    for(i=0;i<600;i++) Delay();
    ReceiveData();
    t--;
  }
}

int main(void){ unsigned char sequenceNum=0;        // bus clock at 50 MHz
  SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                 SYSCTL_XTAL_8MHZ);
  SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);
  SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
  GPIOPinTypeGPIOOutput(GPIO_PORTF_BASE, GPIO_PIN_3+GPIO_PIN_2+GPIO_PIN_0);
  GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_2, 0);
  GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_0, 0);
  GPIOPinTypeGPIOInput(GPIO_PORTE_BASE, GPIO_PIN_3+GPIO_PIN_2+GPIO_PIN_1+GPIO_PIN_0);
  GPIOPadConfigSet(GPIO_PORTE_BASE, GPIO_PIN_3+GPIO_PIN_2+GPIO_PIN_1+GPIO_PIN_0,GPIO_STRENGTH_2MA,GPIO_PIN_TYPE_STD_WPU);
  CAN0_Open();
  EnableInterrupts();
  while(1){
    Delay1ms(51);
    XmtData[0] = GPIOPinRead(GPIO_PORTE_BASE, GPIO_PIN_0);  // 0 or 1
    XmtData[1] = GPIOPinRead(GPIO_PORTE_BASE, GPIO_PIN_1);  // 0 or 2
    XmtData[2] = GPIOPinRead(GPIO_PORTE_BASE, GPIO_PIN_3+GPIO_PIN_2+GPIO_PIN_1+GPIO_PIN_0);  // 0-15
    XmtData[3] = sequenceNum;  // sequence count
    CAN0_SendData(XmtData);
    sequenceNum++;
  }
}
